<?php

namespace Lc\CaracoleBundle\Form\Section;

use App\Repository\Section\SectionStore;
use Lc\CaracoleBundle\Model\Merchant\MerchantInterface;
use Lc\CaracoleBundle\Repository\Section\SectionRepository;
use Lc\CaracoleBundle\Resolver\MerchantResolver;
use Lc\CaracoleBundle\Resolver\SectionResolver;
use Lc\CaracoleBundle\Solver\Section\SectionSolver;
use Lc\SovBundle\Doctrine\EntityManager;
use Lc\SovBundle\Model\User\UserInterface;
use Lc\SovBundle\Translation\TranslatorAdmin;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ButtonType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class SwitchSectionFormType extends AbstractType
{
    protected TranslatorAdmin $translatorAdmin;
    protected SectionStore $sectionStore;
    protected SectionResolver $sectionResolver;
    protected MerchantResolver $merchantResolver;
    protected SectionSolver $sectionSolver;

    public function __construct(
        TranslatorAdmin $translatorAdmin,
        SectionStore $sectionStore,
        SectionResolver $sectionResolver,
        MerchantResolver $merchantResolver,
        SectionSolver   $sectionSolver
    )
    {
        $this->translatorAdmin = $translatorAdmin;
        $this->sectionResolver = $sectionResolver;
        $this->sectionStore = $sectionStore;
        $this->merchantResolver = $merchantResolver;
        $this->sectionSolver = $sectionSolver;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {



//        $builder->add(
//            'id_section',
//            HiddenType::class,
//            [
//                'data' => $section ? $section->getId() : null
//            ]
//        );




        $currentSection = $this->sectionResolver->getCurrent();
        $sections = $this->sectionStore->setMerchant($this->merchantResolver->getCurrent())->getOnline();

        $styleButton = '';
        $classButton = 'btn-section';
        if (null == $currentSection) {
            $classButton .= ' btn-section-current ' . $this->sectionSolver->getHtmlClass($currentSection);
        }
        //TOUT afficher
        $builder->add(
            'switch_all',
            SubmitType::class,
            [
                'label' => 'Tout afficher',
                'attr' => [
                    'class' => $classButton,
                    'style' => $styleButton,
                    'value' => null,
                ]
            ]
        );


        foreach ($sections as $section) {
            $styleButton = '';
            $classButton = 'btn-section';
            if ($section == $currentSection) {
                $classButton .= ' btn-section-current ' . $this->sectionSolver->getHtmlClass($currentSection);
            }

            $builder->add(
                'switch_'.$section->getSlug(),
                SubmitType::class,
                [
                    'label' => $section->getTitle(),
                    'attr' => [
                        'class' => $classButton,
                        'style' => $styleButton,
                        'value' => $section->getId(),
                    ]
                ]
            );
        }
    }

    /**
     * {@inheritdoc}
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(
            [
                'section' => null,
            ]
        );
    }

}
