<?php

namespace Lc\CaracoleBundle\Model\Merchant;


use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Config\TaxRateInterface;
use Lc\CaracoleBundle\Model\PointSale\PointSaleInterface;
use Lc\CaracoleBundle\Model\Section\SectionInterface;
use Lc\CaracoleBundle\Model\Setting\MerchantSettingInterface;
use Lc\SovBundle\Doctrine\Extension\SortableTrait;
use Lc\SovBundle\Doctrine\Pattern\AbstractFullEntity;
use Lc\SovBundle\Model\User\GroupUserInterface;
use Lc\SovBundle\Model\User\UserInterface;

interface MerchantInterface
{
    public function getTitle(): ?string;

    public function setTitle(string $title);

    public function getDescription(): ?string;

    public function setDescription(?string $description);

    public function getCreatedBy(): ?UserInterface;

    public function setCreatedBy(?UserInterface $createdBy);

    public function getUpdatedBy(): ?UserInterface;

    public function setUpdatedBy(?UserInterface $updatedBy);

    public function getDevAlias(): ?string;

    public function setDevAlias(?string $devAlias);

    public function getTaxRate(): ?TaxRateInterface;

    public function setTaxRate(?TaxRateInterface $taxRate): MerchantInterface;

    /**
     * @return Collection|PointSaleInterface[]
     */
    public function getPointSales(): Collection;

    public function addPointSale(PointSaleInterface $pointSale): MerchantInterface;

    public function removePointSale(PointSaleInterface $pointSale): MerchantInterface;

    /**
     * @return Collection|MerchantSettingInterface[]
     */
    public function getSettings(): ?Collection;

    public function addSetting(MerchantSettingInterface $merchantSetting
    ): MerchantInterface;

    public function removeSetting(MerchantSettingInterface $merchantSetting
    ): MerchantInterface;

    public function getAddress(): ?AddressInterface;

    public function setAddress(AddressInterface $address): MerchantInterface;

    /**
     * @return Collection|GroupUserInterface[]
     */
    public function getGroupUsers(): Collection;

    public function addGroupUser(GroupUserInterface $groupUser): MerchantInterface;

    public function removeGroupUser(GroupUserInterface $groupUser): MerchantInterface;

    /**
     * @return Collection|SectionInterface[]
     */
    public function getSections(): ?Collection;

    public function addSection(SectionInterface $section): MerchantInterface;

    public function removeSection(SectionInterface $section): MerchantInterface;

    public function getMetaTitle(): ?string;

    public function setMetaTitle(?string $metaTitle);

    public function getMetaDescription(): ?string;

    public function setMetaDescription(?string $metaDescription);

    public function setOldUrls($oldUrls);

    public function getOldUrls(): ?array;

    public function getSlug(): ?string;

    public function setSlug(?string $slug);

    public function getPosition(): float;

    public function setPosition(float $position);

    public function clearPosition();

    public function getStatus(): ?float;

    public function setStatus(float $status);

    public function getCreatedAt(): ?\DateTimeInterface;

    public function setCreatedAt(\DateTimeInterface $createdAt);

    public function getUpdatedAt(): ?\DateTimeInterface;

    public function setUpdatedAt(\DateTimeInterface $updatedAt);
}