<?php

namespace Lc\CaracoleBundle\Model\Order;

use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Config\TaxRateInterface;
use Lc\CaracoleBundle\Model\File\DocumentInterface;
use Lc\CaracoleBundle\Model\PointSale\PointSaleInterface;
use Lc\CaracoleBundle\Model\Section\SectionInterface;
use Lc\CaracoleBundle\Model\User\VisitorInterface;
use Lc\SovBundle\Model\Ticket\TicketInterface;
use Lc\SovBundle\Model\User\UserInterface;


interface OrderShopInterface
{
    public function getValidationDate(): ?\DateTimeInterface;

    public function setValidationDate(\DateTimeInterface $validationDate
    ): OrderShopInterface;

    public function getUser(): ?UserInterface;

    public function setUser(?UserInterface $user): OrderShopInterface;

    public function getInvoiceAddress(): ?AddressInterface;

    public function setInvoiceAddress(?AddressInterface $invoiceAddress): OrderShopInterface;

    public function getInvoiceAddressText(): ?string;

    public function setInvoiceAddressText(string $invoiceAddressText): OrderShopInterface;

    public function getComment(): ?string;

    public function setComment(?string $comment): OrderShopInterface;

    public function getMeanPayment(): ?string;

    public function setMeanPayment(string $meanPayment): OrderShopInterface;

    /**
     * @return Collection|OrderStatusHistoryInterface[]
     */
    public function getOrderStatusHistories(): Collection;

    public function addOrderStatusHistory(OrderStatusHistoryInterface $orderStatusHistory
    ): OrderShopInterface;

    public function removeOrderStatusHistory(OrderStatusHistoryInterface $orderStatusHistory
    ): OrderShopInterface;

    /**
     * @return Collection|OrderPaymentInterface[]
     */
    public function getOrderPayments($meanPayment = null): Collection;

    public function addOrderPayment(OrderPaymentInterface $orderPayment): OrderShopInterface;

    public function removeOrderPayment(OrderPaymentInterface $orderPayment
    ): OrderShopInterface;

    /**
     * @return Collection|OrderProductInterface[]
     */
    public function getOrderProducts(): Collection;

    public function addOrderProduct(OrderProductInterface $orderProduct): OrderShopInterface;

    public function removeOrderProduct(OrderProductInterface $orderProduct
    ): OrderShopInterface;

    public function getVisitor(): ?VisitorInterface;

    public function setVisitor(?VisitorInterface $visitor): OrderShopInterface;

    public function getDeliveryInfos(): ?string;

    public function setDeliveryInfos(?string $deliveryInfos): OrderShopInterface;

    public function getOrderStatus(): ?OrderStatusInterface;

    public function setOrderStatusProtected(?OrderStatusInterface $orderStatus
    ): OrderShopInterface;

    /**
     * @return Collection|OrderReductionCartInterface[]
     */
    public function getOrderReductionCarts(): Collection;

    public function addOrderReductionCart(OrderReductionCartInterface $orderReductionCart
    ): OrderShopInterface;

    public function removeOrderReductionCart(OrderReductionCartInterface $orderReductionCart
    ): OrderShopInterface;

    /**
     * @return Collection|OrderReductionCreditInterface[]
     */
    public function getOrderReductionCredits(): Collection;

    public function addOrderReductionCredit(OrderReductionCreditInterface $orderReductionCredit
    ): OrderShopInterface;

    public function removeOrderReductionCredit(OrderReductionCreditInterface $orderReductionCredit
    ): OrderShopInterface;

    /**
     * @return Collection|DocumentInterface[]
     */
    public function getDocuments(): Collection;

    public function addDocument(DocumentInterface $document): OrderShopInterface;

    public function removeDocument(DocumentInterface $document): OrderShopInterface;

    /**
     * @return Collection|TicketInterface[]
     */
    public function getTickets(): Collection;

    public function addTicket(TicketInterface $ticket): OrderShopInterface;

    public function removeTicket(TicketInterface $ticket): OrderShopInterface;

    public function getSection(): ?SectionInterface;

    public function setSection(?SectionInterface $section): OrderShopInterface;

    public function getCycleId(): ?int;

    public function setCycleId(?int $cycleId): OrderShopInterface;

    public function getOrderShopCreatedAt(): ?\DateTimeInterface;

    public function setOrderShopCreatedAt(?\DateTimeInterface $orderShopCreatedAt
    ): OrderShopInterface;

    public function getIdValidOrder(): ?int;

    public function setIdValidOrder(?int $idValidOrder): OrderShopInterface;

    public function getDeliveryAddress(): ?AddressInterface;

    public function setDeliveryAddress(?AddressInterface $deliveryAddress
    ): OrderShopInterface;

    public function getDeliveryAddressText(): ?string;

    public function setDeliveryAddressText(string $deliveryAddressText): OrderShopInterface;

    public function getDeliveryPointSale(): ?PointSaleInterface;

    public function setDeliveryPointSale(?PointSaleInterface $deliveryPointSale
    ): OrderShopInterface;

    public function getDeliveryType(): ?string;

    public function setDeliveryType(?string $deliveryType): OrderShopInterface;

    public function getDeliveryPrice(): ?float;

    public function setDeliveryPrice(?float $deliveryPrice): OrderShopInterface;

    public function getDeliveryTaxRate(): ?TaxRateInterface;

    public function setDeliveryTaxRate(?TaxRateInterface $deliveryTaxRate
    ): OrderShopInterface;

    public function getReference(): ?string;

    public function setReference(?string $reference): OrderShopInterface;

    public function getMainOrderShop(): ?self;

    public function setMainOrderShop(?OrderShopModel $mainOrderShop): OrderShopInterface;

    /**
     * @return Collection|OrderShopInterface[]
     */
    public function getComplementaryOrderShops(): Collection;

    public function addComplementaryOrderShop(OrderShopModel $complementaryOrderShop
    ): OrderShopInterface;

    public function removeComplementaryOrderShop(OrderShopModel $complementaryOrderShop
    ): OrderShopInterface;

    public function getDeclineComplementaryOrderShop(): ?bool;

    public function setDeclineComplementaryOrderShop(?bool $declineComplementaryOrderShop
    ): OrderShopInterface;

    public function getOrderAllowByAdmin(): ?bool;

    public function setOrderAllowByAdmin(?bool $orderAllowByAdmin): OrderShopInterface;

    public function getHasReach(): ?int;

    public function setHasReach(?int $hasReach): OrderShopInterface;

    public function getStatTotal(): ?float;

    public function setStatTotal(?float $statTotal): OrderShopInterface;

    public function getStatTotalWithTax(): ?float;

    public function setStatTotalWithTax(?float $statTotalWithTax): OrderShopInterface;

    public function getStatTotalOrderProductsWithReductions(): ?float;

    public function setStatTotalOrderProductsWithReductions(?float $statTotalOrderProductsWithReductions
    ): OrderShopInterface;

    public function getStatTotalOrderProductsWithTaxAndReductions(): ?float;

    public function setStatTotalOrderProductsWithTaxAndReductions(?float $statTotalOrderProductsWithTaxAndReductions
    ): OrderShopInterface;

    public function getStatMarginOrderProductsWithReductions(): ?float;

    public function setStatMarginOrderProductsWithReductions(?float $statMarginOrderProductsWithReductions
    ): OrderShopInterface;

    public function getStatDeliveryPriceWithReduction(): ?float;

    public function setStatDeliveryPriceWithReduction(?float $statDeliveryPriceWithReduction
    ): OrderShopInterface;

    public function getStatDeliveryPriceWithTaxAndReduction(): ?float;

    public function setStatDeliveryPriceWithTaxAndReduction(?float $statDeliveryPriceWithTaxAndReduction
    ): OrderShopInterface;
}
