<?php

namespace Lc\CaracoleBundle\Model\PointSale;


use App\Entity\File\File;
use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Merchant\MerchantInterface;
use Lc\CaracoleBundle\Model\User\UserPointSaleInterface;
use Lc\SovBundle\Model\User\UserInterface;

interface PointSaleInterface
{
    public function getTitle(): ?string;
    public function setTitle(string $title);
    public function getDescription(): ?string;
    public function setDescription(?string $description);
    public function getCreatedBy(): ?UserInterface;
    public function setCreatedBy(?UserInterface $createdBy);
    public function getUpdatedBy(): ?UserInterface;
    public function setUpdatedBy(?UserInterface $updatedBy);
    public function getDevAlias(): ?string;
    public function setDevAlias(?string $devAlias);
    public function getOrderAmountMin(): ?float;
    public function setOrderAmountMin(float $orderAmountMin): PointSaleInterface;

    /**
     * @return Collection|MerchantInterface[]
     */
    public function getMerchants(): Collection;
    public function addMerchant(MerchantInterface $merchant): PointSaleInterface;
    public function removeMerchant(MerchantInterface $merchant): PointSaleInterface;
    public function getCode(): ?string;
    public function setCode(?string $code): PointSaleInterface;
    public function getDeliveryPrice(): ?float;
    public function setDeliveryPrice(float $deliveryPrice): PointSaleInterface;
    public function getIsPublic(): ?bool;
    public function setIsPublic(bool $isPublic): PointSaleInterface;
    public function getAddress(): ?AddressInterface;
    public function setAddress(AddressInterface $address): PointSaleInterface;

    /**
     * @return Collection|UserPointSaleInterface[]
     */
    public function getUserPointSales(): Collection;
    public function addUserPointSale(UserPointSaleInterface $userPointSale
    ): PointSaleInterface;
    public function removeUserPointSale(UserPointSaleInterface $userPointSale
    ): PointSaleInterface;
    public function getImage(): ?File;
    public function setImage(?File $image): PointSaleInterface;
    public function getMetaTitle(): ?string;
    public function setMetaTitle(?string $metaTitle);
    public function getMetaDescription(): ?string;
    public function setMetaDescription(?string $metaDescription);
    public function setOldUrls($oldUrls);
    public function getOldUrls(): ?array;
    public function getSlug(): ?string;
    public function setSlug(?string $slug);

    /**
     * @return float
     */
    public function getPosition(): float;

    /**
     * @param float $position
     * @return $this
     */
    public function setPosition(float $position);
    public function clearPosition();
    public function getStatus(): ?float;
    public function setStatus(float $status);
    public function getCreatedAt(): ?\DateTimeInterface;
    public function setCreatedAt(\DateTimeInterface $createdAt);
    public function getUpdatedAt(): ?\DateTimeInterface;
    public function setUpdatedAt(\DateTimeInterface $updatedAt);

    /**
     * @return Collection|PointSaleSectionInterface[]
     */
    public function getPointSaleSections(): Collection;
    public function addPointSaleSection(PointSaleSectionInterface $pointSaleSection): PointSaleInterface;
    public function removePointSaleSection(PointSaleSectionInterface $pointSaleSection): PointSaleInterface;

}