<?php

namespace Lc\CaracoleBundle\Model\Product;


use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Config\TaxRateInterface;
use Lc\CaracoleBundle\Model\Config\UnitInterface;
use Lc\CaracoleBundle\Model\Reduction\ReductionCatalogInterface;
use Lc\SovBundle\Model\File\FileInterface;
use Lc\SovBundle\Model\User\UserInterface;

interface ProductFamilyInterface
{
    public function getTitle(): ?string;

    public function setTitle(string $title);

    public function getDescription(): ?string;

    public function setDescription(?string $description);

    public function getCreatedBy(): ?UserInterface;

    public function setCreatedBy(?UserInterface $createdBy);

    public function getUpdatedBy(): ?UserInterface;

    public function setUpdatedBy(?UserInterface $updatedBy);

    public function getDevAlias(): ?string;

    public function setDevAlias(?string $devAlias);

    public function getPriceInherited(): ?float;

    public function getUnitInherited(): ?UnitInterface;

    public function getTaxRateInherited(): ?TaxRateInterface;

    public function getBuyingPriceInherited(): ?float;

    public function getBuyingPrice(): ?float;

    public function setBuyingPrice(?float $buyingPrice): ProductFamilyInterface;

    public function getPrice(): ?float;

    public function setPrice(?float $price): ProductFamilyInterface;

    public function getUnit(): ?UnitInterface;

    public function setUnit(?UnitInterface $unit): ProductFamilyInterface;

    public function getTaxRate(): ?TaxRateInterface;

    public function setTaxRate(?TaxRateInterface $taxRate): ProductFamilyInterface;

    public function getActiveProducts(): ?bool;

    public function setActiveProducts(bool $activeProducts): ProductFamilyInterface;

    public function getProductsQuantityAsTitle(): ?bool;

    public function setProductsQuantityAsTitle(bool $productsQuantityAsTitle
    ): ProductFamilyInterface;

    public function getProductsType(): ?string;

    public function setProductsType(?string $productsType): ProductFamilyInterface;

    public function getQuantityLabel(): ?string;

    public function setQuantityLabel(?string $quantityLabel): ProductFamilyInterface;

    /**
     * @return Collection|ProductInterface[]
     */
    public function getProducts(): Collection;

    public function addProduct(ProductInterface $product): ProductFamilyInterface;

    public function removeProduct(ProductInterface $product): ProductFamilyInterface;

    public function getReductionCatalog(): ?ReductionCatalogInterface;

    public function setReductionCatalog(?ReductionCatalogInterface $reductionCatalog
    ): ProductFamilyInterface;

    /**
     * @return Collection|ProductCategoryInterface[]
     */
    public function getProductCategories(): Collection;

    public function initProductCategories();

    public function addProductCategory(ProductCategoryInterface $productCategory
    ): ProductFamilyInterface;

    public function removeProductCategory(ProductCategoryInterface $productCategory
    ): ProductFamilyInterface;

    public function getSubtitle(): ?string;

    public function setSubtitle(?string $subtitle): ProductFamilyInterface;

    public function getWarningMessage(): ?string;

    public function setWarningMessage(?string $warningMessage): ProductFamilyInterface;

    public function getWarningMessageType(): ?string;

    public function setWarningMessageType(?string $warningMessageType
    ): ProductFamilyInterface;

    public function getNote(): ?string;

    public function setNote(?string $note): ProductFamilyInterface;

    public function getBehaviorOutOfStock(): ?string;

    public function setBehaviorOutOfStock(?string $behaviorOutOfStock
    ): ProductFamilyInterface;

    public function getBehaviorCountStock(): ?string;

    public function setBehaviorCountStock(string $behaviorCountStock
    ): ProductFamilyInterface;

    public function getExportTitle(): ?string;

    public function setExportTitle(?string $exportTitle): ProductFamilyInterface;

    public function getExportNote(): ?string;

    public function setExportNote(?string $exportNote): ProductFamilyInterface;

    public function getBehaviorStockCycle(): ?string;

    public function setBehaviorStockCycle(string $behaviorStockCycle
    ): ProductFamilyInterface;

    public function getBehaviorDisplaySale(): ?string;

    public function setBehaviorDisplaySale(string $behaviorDisplaySale
    ): ProductFamilyInterface;

    public function getPropertyNoveltyExpirationDate(): ?\DateTimeInterface;

    public function setPropertyNoveltyExpirationDate(?\DateTimeInterface $propertyNoveltyExpirationDate
    ): ProductFamilyInterface;

    public function getPropertyOrganicLabel(): ?string;

    public function setPropertyOrganicLabel(?string $propertyOrganicLabel
    ): ProductFamilyInterface;

    public function getPropertyAllergens(): ?string;

    public function setPropertyAllergens(?string $propertyAllergens
    ): ProductFamilyInterface;

    public function getPropertyComposition(): ?string;

    public function setPropertyComposition(?string $propertyComposition
    ): ProductFamilyInterface;

    public function getPropertyFragrances(): ?string;

    public function setPropertyFragrances(?string $propertyFragrances
    ): ProductFamilyInterface;

    public function getBehaviorExpirationDate(): ?string;

    public function setBehaviorExpirationDate(?string $behaviorExpirationDate
    ): ProductFamilyInterface;

    public function getTypeExpirationDate(): ?string;

    public function setTypeExpirationDate(?string $typeExpirationDate
    ): ProductFamilyInterface;

    public function getPropertyWeight(): ?string;

    public function setPropertyWeight(?string $propertyWeight): ProductFamilyInterface;

    public function getPropertyQuantity(): ?string;

    public function setPropertyQuantity(?string $propertyQuantity): ProductFamilyInterface;

    public function getPropertyVariety(): ?string;

    public function setPropertyVariety(?string $propertyVariety): ProductFamilyInterface;

    public function getPropertyFeature(): ?string;

    public function setPropertyFeature(?string $propertyFeature): ProductFamilyInterface;

    public function getPropertyAlcoholLevel(): ?string;

    public function setPropertyAlcoholLevel(?string $propertyAlcoholLevel
    ): ProductFamilyInterface;

    public function getPropertyPackaging(): ?string;

    public function setPropertyPackaging(?string $propertyPackaging
    ): ProductFamilyInterface;

    public function getPropertyCharacteristics(): ?string;

    public function setPropertyCharacteristics(?string $propertyCharacteristics
    ): ProductFamilyInterface;

    public function getBehaviorAddToCart(): ?string;

    public function setBehaviorAddToCart(?string $behaviorAddToCart
    ): ProductFamilyInterface;

    public function getBehaviorPrice(): ?string;

    public function setBehaviorPrice(?string $behaviorPrice): ProductFamilyInterface;

    public function getSaleStatus(): ?bool;

    public function setSaleStatus(bool $saleStatus): ProductFamilyInterface;

    public function getImage(): ?FileInterface;

    public function setImage(?FileInterface $image): ProductFamilyInterface;

    /**
     * @return Collection|ProductFamilySectionPropertyInterface[]
     */
    public function getProductFamilySectionProperties(): Collection;

    public function addProductFamilySectionProperty(ProductFamilySectionPropertyInterface $productFamilySectionProperty
    ): ProductFamilyInterface;

    public function removeProductFamilySectionProperty(
        ProductFamilySectionPropertyInterface $productFamilySectionProperty
    ): ProductFamilyInterface;

    /**
     * @return Collection|QualityLabelInterface[]
     */
    public function getQualityLabels(): Collection;

    public function addQualityLabel(QualityLabelInterface $qualityLabel
    ): ProductFamilyInterface;

    public function removeQualityLabel(QualityLabelInterface $qualityLabel
    ): ProductFamilyInterface;

    public function getBuyingPriceByRefUnit(): ?float;

    public function setBuyingPriceByRefUnit(?float $buyingPriceByRefUnit
    ): ProductFamilyInterface;

    public function getPriceByRefUnit(): ?float;

    public function setPriceByRefUnit(?float $priceByRefUnit): ProductFamilyInterface;

    public function getQuantity(): ?float;

    public function setQuantity(?float $quantity): ProductFamilyInterface;

    public function getAvailableQuantity(): ?float;

    public function setAvailableQuantity(?float $availableQuantity
    ): ProductFamilyInterface;

    public function getAvailableQuantityDefault(): ?float;

    public function setAvailableQuantityDefault(?float $availableQuantityDefault
    ): ProductFamilyInterface;

    public function getPropertyExpirationDate(): ?string;

    public function setPropertyExpirationDate(?string $propertyExpirationDate
    ): ProductFamilyInterface;

    public function getMetaTitle(): ?string;

    public function setMetaTitle(?string $metaTitle);

    public function getMetaDescription(): ?string;

    public function setMetaDescription(?string $metaDescription);

    public function setOldUrls($oldUrls);

    public function getOldUrls(): ?array;

    public function getSlug(): ?string;

    public function setSlug(?string $slug);

    public function getPosition(): float;

    public function setPosition(float $position);

    public function clearPosition();

    public function getStatus(): ?float;

    public function setStatus(float $status);

    public function getCreatedAt(): ?\DateTimeInterface;

    public function setCreatedAt(\DateTimeInterface $createdAt);

    public function getUpdatedAt(): ?\DateTimeInterface;

    public function setUpdatedAt(\DateTimeInterface $updatedAt);
}